import {GetCpuUsage, GetVirtualMemory} from "frontend/wailsjs/go/system/InfoUtils";
import {computed, reactive, ref, watch} from "vue";
import {asyncComputed} from "@vueuse/core";
import {bytesToSizeWithUnit} from "src/utils/file/file";

let stat =  reactive({
    cpu : {
        usage: {
            data: [],
            interval: null
        },
    },
    memory: {
        data: {},
        interval: null,
        usage: {
            data: 0,
        }
    }
})


export function useCpuUsage(){
    clearInterval(stat.cpu.usage.interval)

    function loop(){
        GetCpuUsage().then(result=>stat.cpu.usage.data = result)
        stat.cpu.usage.interval = setTimeout(()=>{
            loop()
        },1000)
    }

    loop()
    return computed(()=>stat.cpu.usage.data)
}

function loopMemory(){
    clearInterval(stat.memory.interval)
    function loop(){
        GetVirtualMemory().then(result=> {
            stat.memory.data = result
            stat.memory.usage.data = result.usedPercent
        })
        stat.memory.interval = setTimeout(()=>{
            loop()
        },1000)
    }
    loop()
}

export function useMemoryUsage(){
    if(stat.memory.usage.interval == null){
        loopMemory()
    }

    return computed(()=> Number((stat.memory.usage.data || 0).toFixed(2)))
}

export function useMemoryUsageSize(){
    if(stat.memory.usage.interval == null){
        loopMemory()
    }

    return computed(()=> bytesToSizeWithUnit(stat.memory.data.used || 0))
}

export function useMemoryTotalSize(){
    if(stat.memory.usage.interval == null){
        loopMemory()
    }

    return computed(()=> bytesToSizeWithUnit(stat.memory.data.total || 0))
}


